<?php
defined( 'ABSPATH' ) or die( '' );

define( 'FAMILY_FRIENDLY_USER_LEVEL_GREEN', 'green' );
define( 'FAMILY_FRIENDLY_USER_LEVEL_YELLOW', 'yellow' );
define( 'FAMILY_FRIENDLY_USER_LEVEL_RED', 'red' );
define( 'FAMILY_FRIENDLY_USER_LEVEL_NONE', 'none' );

/**
 * @package family-friendly
 * @version 1.02
 * 
 * Distributed under GNU GENERAL PUBLIC LICENSE - please see license.txt for more details
    Copyright (C) 2018  James Konik
 * 
 */
/*
Plugin Name: Family Friendly
Plugin URI: https://wordpress.org/plugins/family-friendly
Description: Keep things family friendly. Allow users to select age ratings for content
Author: James Konik
Version: 1.03
Author URI: http://jameskonik.com
*/

/**
 * remove comments that trip current filters
 * @param type $comments
 * @param type $post_id
 */
function family_friendly_filter_comments( $comments, $post_id ) {
    if( is_admin() ) {
        return $comments;
    }
    
    //check comment_content and replace it if censored
    $banned_words = family_friendly_get_banned_words_array( family_friendly_get_user_level() );
    foreach( $comments as $comment ) {
        $words = explode ( ' ', strtolower( $comment->comment_content ) );
        foreach( $words as $word ) {
            foreach( $banned_words as $banned_word ) {
                if( $banned_word == $word ) {
                    $comment->comment_content = 'comment removed by family friendly';
                }
            }
        }
        //check comment title and replace if necessary
        $words = explode( ' ', strtolower( $comment->comment_author ) );
        foreach( $words as $word ) {
            foreach( $banned_words as $banned_word ) {
                if( $banned_word == $word ) {
                    $comment->comment_author = 'author name removed by family friendly';
                }
            }
        }
    }
    return $comments;
}

/**
 * function to get banned words list
 */
function family_friendly_get_banned_words_array( $user_level ) {
    $O = get_option( 'family-friendly-options' );
    $banned_words = strtolower( $O['family_friendly_always_banned_words'] );
    if( ( $user_level == FAMILY_FRIENDLY_USER_LEVEL_GREEN ) || ( $user_level == FAMILY_FRIENDLY_USER_LEVEL_YELLOW ) ) {
        if( strlen( $banned_words ) ) {
            $banned_words = $banned_words.',';
        }
        $banned_words = $banned_words.strtolower( $O['family_friendly_secondary_banned_words'] );
    }
    if( $user_level == FAMILY_FRIENDLY_USER_LEVEL_GREEN ) {
        if( strlen( $banned_words ) ) {
            $banned_words = $banned_words.',';
        }
        $banned_words = $banned_words.strtolower( $O['family_friendly_primary_banned_words'] );
    }
    
    //if no words have been found, return a blank array, otherwise the explode function will give us an array of length one, causing problems.
    if( ! strlen( $banned_words ) ) {
        $blank = array();
        return $blank;
    }
    
    $words_array = explode( ',', $banned_words );
        
    $words_array = array_map( 'trim',$words_array );
    
    //add plurals for each word
    $plurals = array();
    foreach( $words_array as $word ) {
        if( $word[strlen($word)-1] != 's' ){
            $newword = $word.'s';
            array_push( $plurals, $newword );
        }
    }
    
    $words_array = array_merge( $words_array, $plurals );
    
    return $words_array;
}


/**
 * Confirm that user level setting is one of recognized values
 */
function family_friendly_is_user_setting_recognized( $input ) {
    $input = sanitize_text_field( preg_replace( '/[^-a-zA-Z0-9_]/', '', $input ) );
    if( ( $input == FAMILY_FRIENDLY_USER_LEVEL_GREEN ) || ( $input == FAMILY_FRIENDLY_USER_LEVEL_NONE ) || ( $input == FAMILY_FRIENDLY_USER_LEVEL_YELLOW ) || ( $input == FAMILY_FRIENDLY_USER_LEVEL_RED ) ) {
        return true;
    }
    return false;
}

/**
 * Determine if the user has picked a filter level
 */
function family_friendly_get_user_level() {
    
    //look for the get setting first of all
    if( isset( $_GET['family_friendly_setting'] ) ) {
      //sanitize input
      $incoming = sanitize_text_field( $_GET['family_friendly_setting'] );
      if( family_friendly_is_user_setting_recognized( $incoming ) ) {
          return $incoming;
      }
    }
    
    //is user level in cookie?
    if ( isset( $_COOKIE['family-friendly'] ) ) {
        $cookie_value = sanitize_text_field( $_COOKIE['family-friendly'] );
        if( family_friendly_is_user_setting_recognized( $cookie_value ) ) {
          return $cookie_value;
        }
    }
    
    //check for default user level here which should be in settings
    $O = get_option( 'family-friendly-options' );
    $default = $O['family_friendly_default_level'];
    if( strlen( $default ) ) {
        return $default;
    }
    
    return FAMILY_FRIENDLY_USER_LEVEL_NONE;
}

/**
 * Count the number of posts that would be filtered by the (parameter supplied) level
 * @param type $level
 * @return type
 */
function family_friendly_count_affected_posts( $level ) {
     $args = array(
        'numberposts'       => -1,
        'orderby'           => 'post_date',
        'order'             => 'DESC',
        'post_type'         => 'post',
        'post_status'       => 'publish',
        'suppress_filters'  => true
    );
    $posts = get_posts( $args );
    
    $banned_words = family_friendly_get_banned_words_array( $level );
    $before = count( $posts );
    $posts = family_friendly_remove_posts_with_banned_words( $posts, $banned_words );
    $after = count( $posts );
    return $before-$after;
}

//issues with double spaced characters having one space converted to a non-standard character
function family_friendly_replace_special_characters( $str ){
    $chunked = str_split( $str,1 );
    $str = ""; 
    foreach( $chunked as $chunk ){
        $num = ord($chunk);
        // Remove non-ascii & non html characters
        if ($num >= 32 && $num <= 123){
                $str.=$chunk;
        }
    }   
    return $str;
} 

function family_friendly_find_word( $needle, $haystack ){
    $haystack = family_friendly_replace_special_characters($haystack);
    $haystack = preg_replace('!\s+!', ' ', $haystack);
    $haystack = explode( ' ', $haystack );
    foreach( $haystack as $word ){
        $tword = trim( $word );
        if( $needle == $tword ){
            return true;
        }
    }
    return false;
}

/**
 * Remove the posts from the array that contain the banned words
 * @param type $posts
 * @param type $level
 */
function family_friendly_remove_posts_with_banned_words( $posts, $banned_words ) {
    $filtered_posts = array();
    //$time_start = microtime(true);
    foreach( $posts as $post ) {
        $is_safe = true;
        $post_tags = get_the_tags( $post->ID );
        
        foreach( $banned_words as $word ) {    
            //check for presence of banned word in title
            if( family_friendly_find_word( $word, strtolower( $post->post_title ) ) === true ) {
                $is_safe = false;
                break;
            }
            
            //check for banned words in content
            if( family_friendly_find_word( $word, strtolower( $post->post_content ) ) === true ) {
                $is_safe = false;
            }
            
            if( $post_tags ) {
                foreach( $post_tags as $tag ) {
                    if( family_friendly_find_word( $word, strtolower( $tag->name ) ) === true ) {
                        $is_safe = false;
                    }
                }
            }
        }
        if( $is_safe ) {
            $filtered_posts[] = $post;
        }
    }
    
    //$time_end = microtime(true);
    //$time = $time_end - $time_start;
    //error_log('function time: '.$time);
    
    return $filtered_posts;
}

/**
 * Filter that removes posts based on current user set level
 * @param type $posts
 * @return type
 */
function family_friendly_filter_posts( $posts ) {
    //the admin should still see banned posts
    if( is_admin() ) {
        return $posts;
    }
     
    $banned_words = family_friendly_get_banned_words_array( family_friendly_get_user_level() );
    
    //if there are no banned words just return the posts here
    if( count( $banned_words ) == 0 ) {
        return $posts;
    }
    
    return family_friendly_remove_posts_with_banned_words( $posts, $banned_words );
}


function family_friendly_admin_actions() {
    add_options_page( 'Family Friendly', 'Family Friendly', 'manage_options', 'Family Friendly', 'family_friendly_admin' );
}



/**
 * validate options
 * @param type $options
 * @return type
 */
function family_friendly_validate_options( $options ) {
        
        $validated['family_friendly_use_green'] = ( isset( $options['family_friendly_use_green'] ) && true == $options['family_friendly_use_green'] ? true : false );
        $validated['family_friendly_use_yellow'] = ( isset( $options['family_friendly_use_yellow'] ) && true == $options['family_friendly_use_yellow'] ? true : false );
        $validated['family_friendly_use_red'] = ( isset( $options['family_friendly_use_red'] ) && true == $options['family_friendly_use_red'] ? true : false );
        
        $validated['family_friendly_cookie_warning'] = ( isset( $options['family_friendly_cookie_warning'] ) && true == $options['family_friendly_cookie_warning'] ? true : false );
        $validated['family_friendly_cookies_disabled_warning'] = ( isset( $options['family_friendly_cookies_disabled_warning'] ) && true == $options['family_friendly_cookies_disabled_warning'] ? true : false );
        $validated['family_friendly_show_widget'] = ( isset( $options['family_friendly_show_widget'] ) && true == $options['family_friendly_show_widget'] ? true : false );
        $validated['family_friendly_show_advanced_settings'] = ( isset( $options['family_friendly_show_advanced_settings'] ) && true == $options['family_friendly_show_advanced_settings'] ? true : false );
        $validated['family_friendly_primary_banned_words'] = sanitize_text_field( $options['family_friendly_primary_banned_words'] );
        $validated['family_friendly_secondary_banned_words'] = sanitize_text_field( $options['family_friendly_secondary_banned_words'] );
        $validated['family_friendly_always_banned_words'] = sanitize_text_field( $options['family_friendly_always_banned_words'] );
        
        $validated['family_friendly_cookie_warning_text'] = sanitize_text_field( $options['family_friendly_cookie_warning_text'] );
        $validated['family_friendly_main_widget_text'] = sanitize_text_field( $options['family_friendly_main_widget_text'] );
        $validated['family_friendly_widget_title'] = sanitize_text_field( $options['family_friendly_widget_title'] );
        $validated['family_friendly_default_level'] = sanitize_text_field( $options['family_friendly_default_level'] );
        
        return $validated;
    }
    
function family_friendly_get_default_options() {
        $options = array(
            'family_friendly_primary_banned_words' => '', // the checkbox-option in question
            'family_friendly_secondary_banned_words' => '',
            'family_friendly_always_banned_words' => '', //words banned in spite of user choice
            'family_friendly_use_green' => 1, //make green option available to users
            'family_friendly_use_yellow' => 1, //make yellow option available to users
            'family_friendly_use_red' => 1, //make red option available to users
            'family_friendly_cookie_warning' => 1,  //display the cookie warning or not
            'family_friendly_cookie_warning_text' => 'Family Friendly uses cookies to store user preferences. Settings will be lost if cookies are cleared or blocked.', //value of cookie warning text
            'family_friendly_cookies_disabled_warning' => 1, //show a warning in widget if cookies are disabled
            'family_friendly_main_widget_text' => 'Select a rating for family friendly to use:',  //main text for cookie widget
            'family_friendly_widget_title' => 'Family Friendly', //widget title
            'family_friendly_show_widget' => 1, //whether users can see the widget or not
            'family_friendly_default_level' => FAMILY_FRIENDLY_USER_LEVEL_NONE, //select a default level for users
            'family_friendly_show_advanced_settings' => 0,
        );
        return $options;
    }

/**
 * store the settings
 */
function family_friendly_register_settings() {
    register_setting( 'family-friendly-options', 'family-friendly-options', array( 'sanitize_callback' => 'family_friendly_validate_options' ) );
        if ( false === ( $options = get_option( 'family-friendly-options' ) ) ) {
            $options = family_friendly_get_default_options();
        }
            
        update_option( 'family-friendly-options', $options );
}


function family_friendly_admin() {
    include( 'family-friendly-options-plugin.php' );
}

//add the admin menu function
add_action( 'admin_menu', 'family_friendly_admin_actions' );
add_action( 'admin_init', 'family_friendly_register_settings' );

//add filters to posts, pages
add_filter( 'posts_results', 'family_friendly_filter_posts', 10, 1 );
add_filter( 'get_pages', 'family_friendly_filter_posts',10,1 );

//add filter to adjust commnets
add_filter( 'comments_array' , 'family_friendly_filter_comments' , 10, 2 ); 

// register widget
add_action( 'widgets_init', 'family_friendly_add_widget' );

add_action( 'init', 'family_friendly_setcookie' );


function family_friendly_add_widget() {
    include( 'family-friendly-widget.php' );
    register_widget( 'Family_Friendly_Widget' );
}

add_action( 'admin_enqueue_scripts', 'family_friendly_admin_add_stylesheet' );

// my_setcookie() set the cookie on the domain and directory WP is installed on
function family_friendly_setcookie() {
  
  $path = parse_url( get_option('siteurl'), PHP_URL_PATH );
  $host = parse_url( get_option('siteurl'), PHP_URL_HOST );
  $expiry = strtotime('+1 month');
  
    if( isset( $_GET['family_friendly_setting'] ) ) {
      
        //sanitize input
        $incoming = sanitize_text_field( $_GET['family_friendly_setting'] );
      
        if( strlen( $incoming ) ) {
            setcookie( 'family-friendly', $incoming, $expiry, $path, $host );
        }
    }
}

function family_friendly_admin_add_stylesheet() {
    //css for admin
    wp_register_style( 'custom_wp_admin_css', plugins_url('css/style.css', __FILE__) );
    wp_enqueue_style( 'custom_wp_admin_css' );
}

?>